@@ -29,6 +29,7 @@ import warnings
 import luigi
 import mock
 import pytest
+from jsonschema.exceptions import ValidationError
 from luigi.freezing import FrozenOrderedDict
 
 import luigi_tools.parameter
@@ -70,6 +71,92 @@ def test_ext_parameter(luigi_tools_working_directory):
         assert luigi.build([TaskExtParameter(test_type="cfg_with_dot")], local_scheduler=True)
 
 
+def test_dict_parameter(luigi_tools_working_directory):
+    """Test the `luigi_tools.parameter.DictParameter` class."""
+
+    class TaskDictParameter(luigi.Task):
+        """A simple test task."""
+
+        a = luigi_tools.parameter.DictParameter(default={"a": 0})
+        b = luigi_tools.parameter.DictParameter(
+            default={"INVALID_ATTRIBUTE": 0},
+            schema={
+                "type": "object",
+                "properties": {
+                    "an_int": {"type": "integer"},
+                    "an_optional_str": {"type": "string"},
+                },
+                "additionalProperties": False,
+                "required": ["an_int"],
+            },
+        )
+
+        def run(self):
+            create_empty_file(self.output().path)
+
+        def output(self):
+            return luigi.LocalTarget(
+                luigi_tools_working_directory
+                / f"test_dict_parameter_{hash(self.a)}_{hash(self.b)}.test"
+            )
+
+    # Check that the default value is validated
+    with pytest.raises(
+        ValidationError,
+        match=r"Additional properties are not allowed \('INVALID_ATTRIBUTE' was unexpected\)",
+    ):
+        luigi.build([TaskDictParameter()], local_scheduler=True)
+
+    # Check that empty dict is not valid
+    empty_dict = {}
+    with pytest.raises(ValidationError, match="'an_int' is a required property"):
+        luigi.build([TaskDictParameter(a=empty_dict, b=empty_dict)], local_scheduler=True)
+
+    # Check that valid dicts work
+    valid_dict = {"an_int": 1}
+    valid_and_complete_dict = {"an_int": 1, "an_optional_str": "hello"}
+    assert luigi.build([TaskDictParameter(a=valid_dict, b=valid_dict)], local_scheduler=True)
+    assert luigi.build(
+        [TaskDictParameter(a=valid_and_complete_dict, b=valid_and_complete_dict)],
+        local_scheduler=True,
+    )
+
+    with set_luigi_config(
+        {"TaskDictParameter": {"a": json.dumps(valid_dict), "b": json.dumps(valid_dict)}}
+    ):
+        assert luigi.build([TaskDictParameter()], local_scheduler=True)
+
+    with set_luigi_config(
+        {
+            "TaskDictParameter": {
+                "a": json.dumps(valid_and_complete_dict),
+                "b": json.dumps(valid_and_complete_dict),
+            }
+        }
+    ):
+        assert luigi.build([TaskDictParameter()], local_scheduler=True)
+
+    # Check that invalid dicts raise correct errors
+    invalid_dict = {"an_int": "999"}
+    invalid_and_complete_dict = {"an_int": 1, "an_optional_str": 999}
+    with set_luigi_config(
+        {"TaskDictParameter": {"a": json.dumps(invalid_dict), "b": json.dumps(invalid_dict)}}
+    ):
+        with pytest.raises(ValidationError, match="'999' is not of type 'integer'"):
+            luigi.build([TaskDictParameter()], local_scheduler=True)
+
+    with set_luigi_config(
+        {
+            "TaskDictParameter": {
+                "a": json.dumps(invalid_and_complete_dict),
+                "b": json.dumps(invalid_and_complete_dict),
+            }
+        }
+    ):
+        with pytest.raises(ValidationError, match="999 is not of type 'string'"):
+            luigi.build([TaskDictParameter()], local_scheduler=True)
+
+
 def test_ratio_parameter(tmpdir):
     """Test the `luigi_tools.parameter.RatioParameter` class."""
 
@@ -1012,6 +1099,48 @@ def test_DataclassParameter__Optional__Union():
     )
 
 
+def test_DataclassParameter__Optional_dataclasses():
+    """Test DataclassParameter with optional dataclass attributes."""
+
+    @dataclasses.dataclass
+    class A:
+        a: int
+
+    @dataclasses.dataclass
+    class B:
+        b: typing.Optional[A] = None
+
+    obj = B(b=A(a=1))
+
+    p = luigi_tools.parameter.DataclassParameter(cls_type=B)
+
+    expected_dict = {"b": {"a": 1}}
+
+    string = p.serialize(obj)
+    assert string == json.dumps(expected_dict)
+
+    dictionary = p.parse(string)
+    assert dictionary == expected_dict
+
+    obj = p.normalize(dictionary)
+    assert isinstance(obj.b, A), type(obj.b)
+
+    obj = B()
+
+    p = luigi_tools.parameter.DataclassParameter(cls_type=B)
+
+    expected_dict = {"b": None}
+
+    string = p.serialize(obj)
+    assert string == json.dumps(expected_dict)
+
+    dictionary = p.parse(string)
+    assert dictionary == expected_dict
+
+    obj = p.normalize(dictionary)
+    assert obj.b is None
+
+
 def test_DataclassParameter__Any():
     """Test the DataclassParameter with typing.Any attributes."""
 
