@@ -13,6 +13,12 @@
 # limitations under the License.
 
 """Tests for luigi-tools tasks."""
+
+# pylint: disable=no-member
+# pylint: disable=protected-access
+# pylint: disable=too-many-lines
+# pylint: disable=unused-argument
+# pylint: disable=unused-variable
 import json
 import logging
 import warnings
@@ -37,10 +43,14 @@ from .tools import create_not_empty_file
 
 
 class TestCopyParams:
+    """Test the `luigi_tools.task.copy_params` decorator."""
+
     @pytest.mark.filterwarnings("ignore::UserWarning:luigi.parameter")
-    def test_copy_params(tmpdir):
+    def test_copy_params(self, tmpdir):
+        """Simple test."""
+
         class TaskA(luigi.Task):
-            """"""
+            """A simple test task."""
 
             a = luigi.Parameter(default="default_value_a")
             b = luigi.Parameter(default="default_value_b")
@@ -59,7 +69,7 @@ class TestCopyParams:
             a_none=luigi_tools.task.ParamRef(TaskA, "a", None),
         )
         class TaskB(luigi.Task):
-            """"""
+            """A simple test task."""
 
             b = luigi.Parameter(default="b")
             b_none = luigi.Parameter(default=None)
@@ -95,7 +105,7 @@ class TestCopyParams:
 
             @luigi_tools.task.copy_params()
             class TaskC(luigi.Task):
-                """"""
+                """A simple test task."""
 
                 a = luigi.Parameter(default="a")
 
@@ -106,13 +116,13 @@ class TestCopyParams:
                 a=luigi_tools.task.ParamRef(TaskA),
             )
             class TaskD(luigi.Task):
-                """"""
+                """A simple test task."""
 
                 a = luigi.Parameter(default="a")
 
         # Test with parameters that are serialized to generate the task ID
         class TaskWithListDictParams(luigi.Task):
-            """"""
+            """A simple test task."""
 
             a = luigi.ListParameter(description="a in TaskWithListDictParams")
             b = luigi.DictParameter(description="b in TaskWithListDictParams")
@@ -129,7 +139,7 @@ class TestCopyParams:
             b_copy=luigi_tools.task.ParamRef(TaskWithListDictParams, "b"),
         )
         class TaskCopyListDictParams(luigi.Task):
-            """"""
+            """A simple test task."""
 
             a = luigi.ListParameter(description="a in TaskCopyListDictParams")
             b = luigi.DictParameter(description="b in TaskCopyListDictParams")
@@ -168,10 +178,10 @@ class TestCopyParams:
         str_new_value,
         **kwargs,
     ):
+        """Test function used for different types."""
 
-        # Test with parameters that are serialized to generate the task ID
         class TaskWithTypeParams(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A task with some parameters of given type."""
 
             a = cls_param(description="a in TaskWithTypeParams")
 
@@ -185,13 +195,13 @@ class TestCopyParams:
             a_copy=luigi_tools.task.ParamRef(TaskWithTypeParams, "a"),
         )
         class TaskCopyTypeParams(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A task that copies some parameters of given type."""
 
             a = cls_param(description="a in TaskCopyTypeParams")
             with_value = luigi.BoolParameter()
 
             def run(self):
-                if cls_param is luigi_tools.parameter.OptionalListParameter:
+                if cls_param is luigi.parameter.OptionalListParameter:
                     assert self.a == tuple(initial_value)
                 else:
                     assert self.a == initial_value
@@ -236,17 +246,21 @@ class TestCopyParams:
         print("test end with", cls_param)
 
     def test_int_param(self, tmp_working_dir):
+        """Test with int parameters."""
         self.type_test(luigi.parameter.IntParameter, 1, "1", 2, "2")
 
     def test_float_param(self, tmp_working_dir):
+        """Test with float parameters."""
         self.type_test(luigi.parameter.FloatParameter, 1.5, "1.5", 2.5, "2.5")
 
     def test_bool_param(self, tmp_working_dir):
+        """Test with bool parameters."""
         self.type_test(luigi_tools.parameter.BoolParameter, True, "true", False, "false")
 
     def test_optional_list_param(self, tmp_working_dir):
+        """Test with optional list parameters."""
         self.type_test(
-            luigi_tools.parameter.OptionalListParameter,
+            luigi.parameter.OptionalListParameter,
             (1, 2),
             "[1, 2]",
             (10, 20),
@@ -255,11 +269,15 @@ class TestCopyParams:
 
 
 class TestCopyParamsWithGlobals:
+    """Test the `luigi_tools.task.copy_params` decorator with global parameters."""
+
     @pytest.mark.filterwarnings("ignore::UserWarning:luigi.parameter")
     @pytest.fixture
     def TaskA(self):
+        """A simple test task."""
+
         class TaskA(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             a = luigi.Parameter(default="a")
             a_cfg = luigi.Parameter(default="a_cfg")
@@ -275,13 +293,15 @@ class TestCopyParamsWithGlobals:
 
     @pytest.mark.filterwarnings("ignore::UserWarning:luigi.parameter")
     def test_defaults(self, luigi_tools_working_directory, TaskA):
+        """Test with default usage."""
+
         @luigi_tools.task.copy_params(
             aa=luigi_tools.task.ParamRef(TaskA, "a_cfg"),
             a_default=luigi_tools.task.ParamRef(TaskA, "a", "given_default_value"),
             a_none=luigi_tools.task.ParamRef(TaskA, "a", None),
         )
         class TaskB(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             b = luigi.Parameter(default="b")
             b_none = luigi.Parameter(default=None)
@@ -330,32 +350,42 @@ class TestCopyParamsWithGlobals:
         )
 
     def test_empty(self):
-        # Empty copy_params arguments should raise a ValueError
+        """Test with empty params.
+
+        Empty copy_params arguments should raise a ValueError.
+        """
         with pytest.raises(ValueError):
 
             @luigi_tools.task.copy_params()
             class TaskC(luigi.Task):
-                """"""
+                """A simple test task."""
 
                 a = luigi.Parameter(default="a")
 
     def test_duplicated(self, TaskA):
-        # Duplicated parameters should raise a DuplicatedParameterError
+        """Test with duplicated params.
+
+        Duplicated parameters should raise a DuplicatedParameterError.
+        """
         with pytest.raises(DuplicatedParameterError):
 
             @luigi_tools.task.copy_params(
                 a=luigi_tools.task.ParamRef(TaskA),
             )
             class TaskD(luigi_tools.task.GlobalParamMixin, luigi.Task):
-                """"""
+                """A simple test task."""
 
                 a = luigi.Parameter(default="a")
 
     @pytest.mark.filterwarnings("ignore::UserWarning:luigi.parameter")
     def test_no_default_value(self):
-        # Global parameter with _no_default_value should raise GlobalParameterNoValueError
+        """Test without default value.
+
+        Global parameter with _no_default_value should raise GlobalParameterNoValueError.
+        """
+
         class TaskE(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             e = luigi.Parameter(default=luigi_tools.task._no_default_value)
 
@@ -370,12 +400,16 @@ class TestCopyParamsWithGlobals:
 
     @pytest.mark.filterwarnings("ignore::UserWarning:luigi.parameter")
     def test_copy_params_no_default_value(self, TaskA):
-        # Global parameter with _no_default_value should raise GlobalParameterNoValueError
+        """Test copied param without default value.
+
+        Global parameter with _no_default_value should raise GlobalParameterNoValueError.
+        """
+
         @luigi_tools.task.copy_params(
             a=luigi_tools.task.ParamRef(TaskA),
         )
         class TaskF(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             f = luigi.Parameter(default=luigi_tools.task._no_default_value)
 
@@ -390,9 +424,10 @@ class TestCopyParamsWithGlobals:
             luigi.build([TaskF()], local_scheduler=True)
 
     def test_inherits(self):
-        # Compare with luigi.util.inherits
+        """Compare with luigi.util.inherits."""
+
         class TaskG(luigi.Task):
-            """"""
+            """A simple test task."""
 
             g = luigi.Parameter(default="default_value_g")
 
@@ -404,7 +439,7 @@ class TestCopyParamsWithGlobals:
 
         @inherits(TaskG)
         class TaskH(luigi.Task):
-            """"""
+            """A simple test task."""
 
             h = luigi.Parameter()
 
@@ -446,9 +481,10 @@ class TestCopyParamsWithGlobals:
             ]
 
     def test_compare_inherits(self, luigi_tools_working_directory):
-        # Compare with luigi.util.inherits
+        """Compare with luigi.util.inherits."""
+
         class TaskI(luigi.Task):
-            """"""
+            """A simple test task."""
 
             i = luigi.Parameter(default="default_value_i")
 
@@ -463,7 +499,7 @@ class TestCopyParamsWithGlobals:
             i=luigi_tools.task.ParamRef(TaskI),
         )
         class TaskJ(luigi.Task):
-            """"""
+            """A simple test task."""
 
             j = luigi.Parameter()
 
@@ -493,9 +529,10 @@ class TestCopyParamsWithGlobals:
             assert luigi.build([TaskJ()], local_scheduler=True)
 
     def test_serialized_parameters(self):
-        # Test with parameters that are serialized to generate the task ID
+        """Test with parameters that are serialized to generate the task ID."""
+
         class GlobalParamTaskWithListDictParams(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             a = luigi.ListParameter(default="a in GlobalParamTaskWithListDictParams")
             b = luigi.DictParameter(default="b in GlobalParamTaskWithListDictParams")
@@ -512,7 +549,7 @@ class TestCopyParamsWithGlobals:
             b_copy=luigi_tools.task.ParamRef(GlobalParamTaskWithListDictParams, "b"),
         )
         class GlobalParamTaskCopyListDictParams(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             a_new = luigi.ListParameter(default="a in GlobalParamTaskCopyListDictParams")
             b_new = luigi.DictParameter(default="b in GlobalParamTaskCopyListDictParams")
@@ -547,11 +584,17 @@ class TestCopyParamsWithGlobals:
             )
 
     def test_not_comparable_attribute(self):
+        """Test with params that can't be compared."""
+
         class NotComparable:
+            """A class that always raises an exception when __eq__ is called."""
+
             def __eq__(self, other):
                 raise TypeError(f"Can't compare {self.__class__} with {type(other)}")
 
         class GlobalParamTaskSetGetAttr(luigi_tools.task.GlobalParamMixin, luigi.Task):
+            """A simple test task."""
+
             a = luigi.ListParameter(default="a in GlobalParamTaskSetGetAttr")
             b = None
             c = 1
@@ -590,7 +633,7 @@ class TestCopyParamsWithGlobals:
         """
 
         class TaskListParameter(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             a = luigi.parameter.ListParameter(default=(1, 2))
             b = luigi.parameter.ListParameter()
@@ -607,7 +650,7 @@ class TestCopyParamsWithGlobals:
             b=luigi_tools.task.ParamRef(TaskListParameter),
         )
         class TaskCopyListParameter(luigi_tools.task.GlobalParamMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             def run(self):
                 assert self.a == (1, 2)
@@ -621,29 +664,35 @@ class TestCopyParamsWithGlobals:
 
 
 class TestForceableTask:
+    """Test the rerun feature."""
+
     def test_no_rerun(self, task_collection):
-        # Test that everything is run when all rerun are False and targets are missing
+        """Test that everything is run when all rerun are False and targets are missing."""
         assert all(
-            [not check_existing_file(i.path) for i in luigi.task.flatten(task_collection.targets)]
+            not check_existing_file(i.path) for i in luigi.task.flatten(task_collection.targets)
         )
 
         assert luigi.build([task_collection.TaskE()], local_scheduler=True)
 
         assert all(
-            [check_not_empty_file(i.path) for i in luigi.task.flatten(task_collection.targets)]
+            check_not_empty_file(i.path) for i in luigi.task.flatten(task_collection.targets)
         )
 
     def test_no_rerun_with_complete_targets(self, task_collection):
-        # Test that nothing is run when all rerun are False and targets are present
+        """Test that nothing is run when all rerun are False and targets are present."""
         for i in luigi.task.flatten(task_collection.targets):
             create_empty_file(i.path)
 
         assert luigi.build([task_collection.TaskE()], local_scheduler=True)
 
-        assert all([check_empty_file(i.path) for i in luigi.task.flatten(task_collection.targets)])
+        assert all(check_empty_file(i.path) for i in luigi.task.flatten(task_collection.targets))
 
     def test_rerun_with_complete_targets(self, task_collection):
-        # Test that everything is run when rerun = True for the root task and targets are present
+        """Test that everything is run when rerun = True for the root task.
+
+        The targets of all tasks are present and should thus be automatically removed before running
+        the workflow.
+        """
         for i in luigi.task.flatten(task_collection.targets):
             create_empty_file(i.path)
 
@@ -655,11 +704,11 @@ class TestForceableTask:
             assert luigi.build([task_collection.TaskE()], local_scheduler=True)
 
         assert all(
-            [check_not_empty_file(i.path) for i in luigi.task.flatten(task_collection.targets)]
+            check_not_empty_file(i.path) for i in luigi.task.flatten(task_collection.targets)
         )
 
     def test_rerun_parents_only(self, task_collection):
-        # Test that only the parents of the task with rerun = True are run
+        """Test that only the parents of the task with rerun = True are run."""
         for i in luigi.task.flatten(task_collection.targets):
             create_empty_file(i.path)
 
@@ -671,27 +720,23 @@ class TestForceableTask:
             assert luigi.build([task_collection.TaskE()], local_scheduler=True)
 
         assert all(
-            [
-                check_not_empty_file(i.path)
-                for task_name, targets in task_collection.targets.items()
-                for j in luigi.task.flatten(targets)
-                if task_name not in ["TaskA", "TaskC"]
-            ]
+            check_not_empty_file(j.path)
+            for task_name, targets in task_collection.targets.items()
+            for j in luigi.task.flatten(targets)
+            if task_name not in ["TaskA", "TaskC"]
         )
         assert all(
-            [
-                check_empty_file(j.path)
-                for task_name, targets in task_collection.targets.items()
-                for j in luigi.task.flatten(targets)
-                if task_name in ["TaskA", "TaskC"]
-            ]
+            check_empty_file(j.path)
+            for task_name, targets in task_collection.targets.items()
+            for j in luigi.task.flatten(targets)
+            if task_name in ["TaskA", "TaskC"]
         )
 
     def test_no_remove_if_recall(self, task_collection, tmpdir):
-        # Test that calling a task inside another one does not remove its targets
+        """Test that calling a task inside another one does not remove its targets."""
 
         class TaskF(luigi_tools.task.WorkflowTask):
-            """"""
+            """A simple test task."""
 
             counter = luigi.IntParameter(default=0)
             rerun = luigi.BoolParameter()
@@ -726,27 +771,27 @@ class TestForceableTask:
             assert luigi.build([TaskF()], local_scheduler=True)
 
         assert all(
-            [
-                check_not_empty_file(i.path)
-                for task_name, targets in task_collection.targets.items()
-                for j in luigi.task.flatten(targets)
-                if task_name not in ["TaskA", "TaskC"]
-            ]
+            check_not_empty_file(j.path)
+            for task_name, targets in task_collection.targets.items()
+            for j in luigi.task.flatten(targets)
+            if task_name not in ["TaskA", "TaskC"]
         )
         assert all(
-            [
-                check_empty_file(j.path)
-                for task_name, targets in task_collection.targets.items()
-                for j in luigi.task.flatten(targets)
-                if task_name in ["TaskA", "TaskC"]
-            ]
+            check_empty_file(j.path)
+            for task_name, targets in task_collection.targets.items()
+            for j in luigi.task.flatten(targets)
+            if task_name in ["TaskA", "TaskC"]
         )
 
 
 class TestLogTargetMixin:
+    """Test the `luigi_tools.task.LogTargetMixin` class."""
+
     def test_output_logger(self, tmpdir, caplog):
+        """Test the logger outputs with one target."""
+
         class TaskA(luigi_tools.task.LogTargetMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             a = luigi.Parameter(default="a")
 
@@ -763,12 +808,16 @@ class TestLogTargetMixin:
         assert res == [("luigi_tools.task", 10, f"Output of TaskA task: {tmpdir / 'test_a'}")]
 
     def test_output_logger_with_dict(self, tmpdir, caplog):
+        """Test the logger outputs with a dict target."""
+
         class AlwaysExistingTarget(luigi.Target):
+            """A target that is always considered as existing."""
+
             def exists(self):
                 return True
 
         class TaskB(luigi_tools.task.LogTargetMixin, luigi.Task):
-            """"""
+            """A simple test task."""
 
             b = luigi.Parameter(default="b")
 
@@ -788,7 +837,7 @@ class TestLogTargetMixin:
         assert res == [("luigi_tools.task", 10, f"Output b of TaskB task: {tmpdir / 'test_b'}")]
 
         class TaskC(luigi.Task):
-            """"""
+            """A simple test task."""
 
             c = luigi.Parameter(default="c")
 
@@ -806,8 +855,10 @@ class TestLogTargetMixin:
         assert res == []
 
     def test_output_logger_ignored(self, tmpdir, caplog):
+        """Test that there is no logger output when the mixin is located after `luigi.Task`."""
+
         class TaskD(luigi.Task, luigi_tools.task.LogTargetMixin):
-            """"""
+            """A simple test task."""
 
             d = luigi.Parameter(default="d")
 
@@ -825,12 +876,14 @@ class TestLogTargetMixin:
         assert res == []
 
     def test_output_logger_and_global_param(self, tmpdir, caplog):
+        """Test logger outputs when interacting with `luigi_tools.task.GlobalParamMixin`."""
+
         class TaskE(
             luigi_tools.task.LogTargetMixin,
             luigi_tools.task.GlobalParamMixin,
             luigi.Task,
         ):
-            """"""
+            """A simple test task."""
 
             e = luigi.Parameter(default="e")
 
@@ -857,6 +910,8 @@ class TestLogTargetMixin:
 
 
 def test_remove_corrupted_output(tmpdir, caplog):
+    """Test the `luigi_tools.task.RemoveCorruptedOutputMixin` mixin."""
+
     class TaskToFail(luigi_tools.task.RemoveCorruptedOutputMixin, luigi.Task):
         """A Task that's expected to fail and create incomplete/wrong output."""
 
@@ -865,7 +920,7 @@ def test_remove_corrupted_output(tmpdir, caplog):
 
         def run(self):
             f_path = self.output().path
-            with open(f_path, "w") as f_handle:
+            with open(f_path, "w", encoding="utf-8") as f_handle:
                 for idx in range(10):
                     f_handle.write(f"{idx}\n")
                     if idx >= 5:
@@ -898,15 +953,19 @@ def test_remove_corrupted_output(tmpdir, caplog):
     # all targets are produced
     assert task_instance.complete()
 
-    with open(tmpdir / "matrix.dat", "r") as f_handle:
+    with open(tmpdir / "matrix.dat", "r", encoding="utf-8") as f_handle:
         matrix_values = f_handle.read().splitlines()
         assert len(matrix_values) == 6
 
 
-class TestCheckUnconsumedParamsMixin:
+class TestCheckUnconsumedParams:
+    """Check that warnings are properly emitted when a parameter is not consumed by a task."""
+
     def test_unconsumed(self, tmpdir, caplog):
+        """Simple test with unconsumed parameters."""
+
         class TaskA(luigi_tools.task.WorkflowTask):
-            """"""
+            """A simple test task."""
 
             a = luigi.Parameter(default="a")
 
@@ -917,7 +976,7 @@ class TestCheckUnconsumedParamsMixin:
                 return luigi.LocalTarget(tmpdir / f"test_TaskA_{self.a}")
 
         class TaskB(luigi_tools.task.WorkflowTask):
-            """"""
+            """A simple test task."""
 
             a = luigi.Parameter(default="a")
 
@@ -948,7 +1007,7 @@ class TestCheckUnconsumedParamsMixin:
                 )
                 warnings.simplefilter(
                     action="always",
-                    category=luigi_tools.parameter.UnconsumedParameterWarning,
+                    category=luigi.parameter.UnconsumedParameterWarning,
                 )
                 assert luigi.build([TaskA(), TaskB()], local_scheduler=True)
 
@@ -960,11 +1019,11 @@ class TestCheckUnconsumedParamsMixin:
                     ("c", "TaskB"),
                 ]
                 for i, (expected_value, task_name) in zip(w, expected):
-                    assert issubclass(i.category, luigi_tools.parameter.UnconsumedParameterWarning)
+                    assert issubclass(i.category, luigi.parameter.UnconsumedParameterWarning)
                     assert str(i.message) == (
                         "The configuration contains the parameter "
                         f"'{expected_value}' with value '{expected_value}' that is not consumed by "
                         f"the task '{task_name}'."
                     )
 
-        assert all([check_not_empty_file(i.path) for i in luigi.task.flatten(TaskA().output())])
+        assert all(check_not_empty_file(i.path) for i in luigi.task.flatten(TaskA().output()))
